Imports Microsoft.VisualBasic
Imports System
Imports System.Data
Imports System.Configuration
Imports System.Web
Imports System.Web.Security
Imports System.Web.UI
Imports System.Web.UI.WebControls
Imports System.Web.UI.WebControls.WebParts
Imports System.Web.UI.HtmlControls
Imports System.ComponentModel
Imports System.Drawing
Imports System.Web.SessionState
Imports System.IO
Imports System.Runtime.Serialization
Imports System.Runtime.Serialization.Formatters
Imports System.Runtime.Serialization.Formatters.Soap
Imports System.Runtime.Serialization.Formatters.Binary

Imports Syncfusion.Windows.Forms.Diagram

Public partial Class FlowDiagram : Inherits System.Web.UI.Page
	Protected Sub Page_Load(ByVal sender As Object, ByVal e As EventArgs)
		If Me.IsPostBack = False Then
			CreateFlowDiagram()
			' Enable the diagram for node selection and drag-drop
			Me.DiagramWebControl1.EnableNodeSelection = True
			Me.DiagramWebControl1.EnableDragDrop = True
		End If
		Me.DiagramWebControl1.RenderImageMap = True

	End Sub

	Private Sub CreateFlowDiagram()
		Dim palettepath As String = Server.MapPath(String.Empty)
		palettepath = palettepath & "\App_Data\FlowDiagramSymbols.edp"
		Dim palette As SymbolPalette = Me.LoadPalette(palettepath)

		' Add a TextNode to highlight the subnode feature
		Dim text As String = "Flow Process Diagram"
		Dim units As MeasureUnits = MeasureUnits.Pixel
		Dim txtnode As Syncfusion.Windows.Forms.Diagram.TextNode = New TextNode(text)
		txtnode.FontStyle.Size = 18
		txtnode.FontStyle.Bold = True
		txtnode.FontStyle.Family = "Arial"
		txtnode.FontColorStyle.Color = Color.Black
		txtnode.LineStyle.LineColor = Color.Transparent
		' for update size.
		txtnode.Text = text
		txtnode.SizeToText(New SizeF(500, 500))
		CType(txtnode, IUnitIndependent).SetPinPoint(New PointF(285, 30), units)
		Me.DiagramWebControl1.Model.AppendChild(txtnode)

		AddNode(palette, "Start", New PointF(50, 100))
		AddNode(palette, "Process", New PointF(185, 100))
		AddNode(palette, "Decision", New PointF(185, 200))
		AddNode(palette, "Document", New PointF(185, 400))
		AddNode(palette, "End", New PointF(320, 460))

		Dim node As Node = GetPaletteNode(palette, "Process")
		node = CType(node.Clone(), Node)
        CType(node, PathNode).Labels(0).Text = "Post-Process1"
		AppendNodeToDocument(node, New PointF(185, 300))

		ConnectNodes(GetModelNode("Start"), GetModelNode("Process"))
        ConnectNodes(GetModelNode("Process"), GetModelNode("Decision"))

		ConnectNodes(GetModelNode("Decision"), node)
		ConnectNodes(node, GetModelNode("Document"))
		ConnectNodes(GetModelNode("Document"), GetModelNode("End"))
	End Sub

	Private Sub ConnectNodes(ByVal node1 As Node, ByVal node2 As Node)
		If node1 Is Nothing Then
		Throw New ArgumentNullException("node1")
		End If
		If node2 Is Nothing Then
		Throw New ArgumentNullException("node2")
		End If

		' enable center port
		node1.EnableCentralPort = True
		node2.EnableCentralPort = True

		' create connector
		Dim conn As LineConnector = New LineConnector(New PointF(1, 1), New PointF(2, 2))

		' set decorator
		Dim decor As Decorator = conn.HeadDecorator
		decor.DecoratorShape = DecoratorShape.FilledFancyArrow
		decor.FillStyle.Color = Color.Black

		Me.DiagramWebControl1.Model.AppendChild(conn)
		' connect head end point
		Dim endPoint As EndPoint = conn.TailEndPoint
		node1.CentralPort.TryConnect(endPoint)
		' connect tail end point
		endPoint = conn.HeadEndPoint
		node2.CentralPort.TryConnect(endPoint)
	End Sub

	Private Sub AddNode(ByVal palette As SymbolPalette, ByVal strNodeName As String, ByVal ptLocation As PointF)
		If palette Is Nothing Then
		Throw New ArgumentNullException("palette")
		End If
		If strNodeName Is Nothing Then
		Throw New ArgumentNullException("strNodeName")
		End If

		Dim nodeTemp As Node = GetPaletteNode(palette, strNodeName)

		If Not nodeTemp Is Nothing Then
			AppendNodeToDocument(nodeTemp, ptLocation)
		End If
	End Sub
	Private Sub AppendNodeToDocument(ByVal node As Node, ByVal ptLocation As PointF)
		If node Is Nothing Then
		Throw New ArgumentNullException("node")
		End If

		Dim szPinOffset As SizeF = (CType(node, IUnitIndependent)).GetPinPointOffset(MeasureUnits.Pixel)
		CType(node, IUnitIndependent).SetPinPoint(New PointF(ptLocation.X + szPinOffset.Width, ptLocation.Y + szPinOffset.Height), MeasureUnits.Pixel)
		Me.DiagramWebControl1.Model.AppendChild(node)
	End Sub
	' Get the SymbolPalette by deserializing the SymbolPalette file
	Public Function LoadPalette(ByVal filename As String) As SymbolPalette
		Dim curSymbolPalette As SymbolPalette = Nothing
		Dim iStream As FileStream = New FileStream(filename, FileMode.Open, FileAccess.Read)

		If Not iStream Is Nothing Then
			Dim formatter As IFormatter = New BinaryFormatter()
			Try
				AddHandler AppDomain.CurrentDomain.AssemblyResolve, AddressOf Syncfusion.DiagramBaseAssembly.AssemblyResolver
				curSymbolPalette = CType(formatter.Deserialize(iStream), SymbolPalette)
			Catch e1 As SerializationException
				Try
					formatter = New SoapFormatter()
					iStream.Position = 0
					curSymbolPalette = CType(formatter.Deserialize(iStream), SymbolPalette)
				Catch e As Exception
					System.Diagnostics.Trace.WriteLine("Error reading SymbolPalette", e.Message)
				End Try
			Finally
				iStream.Close()
			End Try
		End If

		Return curSymbolPalette
	End Function
	Public Function GetModelNode(ByVal strNodeName As String) As Node
		If strNodeName Is Nothing Then
			Throw New ArgumentNullException("strNodeName")
		End If

		Dim nodeToReturn As Node = Nothing
		Dim model As Model = Me.DiagramWebControl1.Model

		If Not model Is Nothing Then
			For Each node As Node In model.Nodes
				If node.Name = strNodeName Then
					nodeToReturn = node
					Exit For
				End If
			Next node
		End If

		Return nodeToReturn
	End Function
	' Get the SymbolModel from the SymbolPalette
	Public Function GetPaletteNode(ByVal palette As SymbolPalette, ByVal symbolName As String) As Node
		If palette Is Nothing Then
			Throw New ArgumentNullException("palette")
		End If

		If symbolName Is Nothing Then
			Throw New ArgumentNullException("symbolName")
		End If

		Dim nodeToReturn As Node = Nothing

		For Each node As Node In palette.Nodes
			If node.Name = symbolName Then
				nodeToReturn = node
				Exit For
			End If
		Next node

		Return nodeToReturn
	End Function
End Class
